@@ -1,36 +1,40 @@
-# coding: utf-8
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import collections
 import hashlib
 import itertools
 import logging
+import optparse
 import os
 from contextlib import contextmanager
 from shutil import rmtree
+from typing import Any, ContextManager, Dict, Iterator, List, Optional, Set, cast
 
+from click import progressbar
 from pip._internal.cache import WheelCache
 from pip._internal.cli.progress_bars import BAR_TYPES
 from pip._internal.commands import create_command
-from pip._internal.models.index import PackageIndex, PyPI
+from pip._internal.commands.install import InstallCommand
+from pip._internal.index.package_finder import PackageFinder
+from pip._internal.models.candidate import InstallationCandidate
+from pip._internal.models.index import PackageIndex
 from pip._internal.models.link import Link
 from pip._internal.models.wheel import Wheel
-from pip._internal.req import RequirementSet
+from pip._internal.network.session import PipSession
+from pip._internal.req import InstallRequirement, RequirementSet
 from pip._internal.req.req_tracker import get_requirement_tracker
 from pip._internal.utils.hashes import FAVORITE_HASH
 from pip._internal.utils.logging import indent_log, setup_logging
 from pip._internal.utils.misc import normalize_path
 from pip._internal.utils.temp_dir import TempDirectory, global_tempdir_manager
 from pip._internal.utils.urls import path_to_url, url_to_path
-from pip._vendor.requests import RequestException
+from pip._vendor.packaging.tags import Tag
+from pip._vendor.packaging.version import _BaseVersion
+from pip._vendor.requests import RequestException, Session
 
-from .._compat import PIP_VERSION, TemporaryDirectory, contextlib
-from ..click import progressbar
+from .._compat import contextlib
 from ..exceptions import NoCandidateFound
 from ..logging import log
 from ..utils import (
     as_tuple,
-    fs_str,
     is_pinned_requirement,
     is_url_requirement,
     lookup_table,
@@ -43,7 +47,6 @@ FileStream = collections.namedtuple("FileStream", "stream size")
 
 
 class PyPIRepository(BaseRepository):
-    DEFAULT_INDEX_URL = PyPI.simple_url
     HASHABLE_PACKAGE_TYPES = {"bdist_wheel", "sdist"}
 
     """
@@ -53,85 +56,75 @@ class PyPIRepository(BaseRepository):
     changed/configured on the Finder.
     """
 
-    def __init__(self, pip_args, cache_dir):
+    def __init__(self, pip_args: List[str], cache_dir: str):
         # Use pip's parser for pip.conf management and defaults.
         # General options (find_links, index_url, extra_index_url, trusted_host,
         # and pre) are deferred to pip.
-        self.command = create_command("install")
-        extra_pip_args = (
-            []
-            if PIP_VERSION[:2] <= (20, 2)
-            else ["--use-deprecated", "legacy-resolver"]
-        )
-        self.options, _ = self.command.parse_args(pip_args + extra_pip_args)
-        if self.options.cache_dir:
-            self.options.cache_dir = normalize_path(self.options.cache_dir)
+        self.command: InstallCommand = create_command("install")
+        extra_pip_args = ["--use-deprecated", "legacy-resolver"]
 
-        self.options.require_hashes = False
-        self.options.ignore_dependencies = False
+        options, _ = self.command.parse_args(pip_args + extra_pip_args)
+        if options.cache_dir:
+            options.cache_dir = normalize_path(options.cache_dir)
+        options.require_hashes = False
+        options.ignore_dependencies = False
 
-        self.session = self.command._build_session(self.options)
-        self.finder = self.command._build_package_finder(
-            options=self.options, session=self.session
+        self._options: optparse.Values = options
+        self._session = self.command._build_session(options)
+        self._finder = self.command._build_package_finder(
+            options=options, session=self.session
         )
 
         # Caches
         # stores project_name => InstallationCandidate mappings for all
         # versions reported by PyPI, so we only have to ask once for each
         # project
-        self._available_candidates_cache = {}
+        self._available_candidates_cache: Dict[str, List[InstallationCandidate]] = {}
 
         # stores InstallRequirement => list(InstallRequirement) mappings
         # of all secondary dependencies for the given requirement, so we
         # only have to go to disk once for each requirement
-        self._dependencies_cache = {}
+        self._dependencies_cache: Dict[InstallRequirement, Set[InstallRequirement]] = {}
 
         # Setup file paths
-        self.freshen_build_caches()
-        self._cache_dir = normalize_path(cache_dir)
-        self._download_dir = fs_str(os.path.join(self._cache_dir, "pkgs"))
-        if PIP_VERSION[:2] <= (20, 2):
-            self._wheel_download_dir = fs_str(os.path.join(self._cache_dir, "wheels"))
+        self._cache_dir = normalize_path(str(cache_dir))
+        self._download_dir = os.path.join(self._cache_dir, "pkgs")
 
         self._setup_logging()
 
-    def freshen_build_caches(self):
-        """
-        Start with fresh build/source caches.  Will remove any old build
-        caches from disk automatically.
-        """
-        self._build_dir = TemporaryDirectory(fs_str("build"))
-        self._source_dir = TemporaryDirectory(fs_str("source"))
-
-    @property
-    def build_dir(self):
-        return self._build_dir.name
-
-    @property
-    def source_dir(self):
-        return self._source_dir.name
-
-    def clear_caches(self):
+    def clear_caches(self) -> None:
         rmtree(self._download_dir, ignore_errors=True)
-        if PIP_VERSION[:2] <= (20, 2):
-            rmtree(self._wheel_download_dir, ignore_errors=True)
 
-    def find_all_candidates(self, req_name):
+    @property
+    def options(self) -> optparse.Values:
+        return self._options
+
+    @property
+    def session(self) -> PipSession:
+        return self._session
+
+    @property
+    def finder(self) -> PackageFinder:
+        return self._finder
+
+    def find_all_candidates(self, req_name: str) -> List[InstallationCandidate]:
         if req_name not in self._available_candidates_cache:
             candidates = self.finder.find_all_candidates(req_name)
             self._available_candidates_cache[req_name] = candidates
         return self._available_candidates_cache[req_name]
 
-    def find_best_match(self, ireq, prereleases=None):
+    def find_best_match(
+        self, ireq: InstallRequirement, prereleases: Optional[bool] = None
+    ) -> InstallRequirement:
         """
-        Returns a Version object that indicates the best match for the given
-        InstallRequirement according to the external repository.
+        Returns a pinned InstallRequirement object that indicates the best match
+        for the given InstallRequirement according to the external repository.
         """
         if ireq.editable or is_url_requirement(ireq):
             return ireq  # return itself as the best match
 
         all_candidates = self.find_all_candidates(ireq.name)
-        candidates_by_version = lookup_table(all_candidates, key=lambda c: c.version)
+        candidates_by_version = lookup_table(all_candidates, key=candidate_version)
         matching_versions = ireq.specifier.filter(
             (candidate.version for candidate in all_candidates), prereleases=prereleases
         )
@@ -152,32 +145,31 @@ class PyPIRepository(BaseRepository):
         return make_install_requirement(
             best_candidate.name,
             best_candidate.version,
-            ireq.extras,
-            constraint=ireq.constraint,
+            ireq,
         )
 
-    def resolve_reqs(self, download_dir, ireq, wheel_cache):
+    def resolve_reqs(
+        self,
+        download_dir: Optional[str],
+        ireq: InstallRequirement,
+        wheel_cache: WheelCache,
+    ) -> Set[InstallationCandidate]:
         with get_requirement_tracker() as req_tracker, TempDirectory(
             kind="resolver"
         ) as temp_dir, indent_log():
-            preparer_kwargs = dict(
-                temp_build_dir=temp_dir,
-                options=self.options,
-                req_tracker=req_tracker,
-                session=self.session,
-                finder=self.finder,
-                use_user_site=False,
-                download_dir=download_dir,
-            )
-            if PIP_VERSION[:2] <= (20, 2):
-                preparer_kwargs["wheel_download_dir"] = self._wheel_download_dir
+            preparer_kwargs = {
+                "temp_build_dir": temp_dir,
+                "options": self.options,
+                "req_tracker": req_tracker,
+                "session": self.session,
+                "finder": self.finder,
+                "use_user_site": False,
+                "download_dir": download_dir,
+            }
             preparer = self.command.make_requirement_preparer(**preparer_kwargs)
 
             reqset = RequirementSet()
-            if PIP_VERSION[:2] <= (20, 1):
-                ireq.is_direct = True
-            else:
-                ireq.user_supplied = True
+            ireq.user_supplied = True
             reqset.add_requirement(ireq)
 
             resolver = self.command.make_resolver(
@@ -195,14 +187,11 @@ class PyPIRepository(BaseRepository):
             if not ireq.prepared:
                 # If still not prepared, e.g. a constraint, do enough to assign
                 # the ireq a name:
-                if PIP_VERSION[:2] <= (20, 2):
-                    resolver._get_abstract_dist_for(ireq)
-                else:
-                    resolver._get_dist_for(ireq)
+                resolver._get_dist_for(ireq)
 
         return set(results)
 
-    def get_dependencies(self, ireq):
+    def get_dependencies(self, ireq: InstallRequirement) -> Set[InstallRequirement]:
         """
         Given a pinned, URL, or editable InstallRequirement, returns a set of
         dependencies (also InstallRequirements, but not necessarily pinned).
@@ -212,16 +201,14 @@ class PyPIRepository(BaseRepository):
             ireq.editable or is_url_requirement(ireq) or is_pinned_requirement(ireq)
         ):
             raise TypeError(
-                "Expected url, pinned or editable InstallRequirement, got {}".format(
-                    ireq
-                )
+                f"Expected url, pinned or editable InstallRequirement, got {ireq}"
             )
 
         if ireq not in self._dependencies_cache:
             if ireq.editable and (ireq.source_dir and os.path.exists(ireq.source_dir)):
                 # No download_dir for locally available editable requirements.
-                # If a download_dir is passed, pip will  unnecessarely
-                # archive the entire source directory
+                # If a download_dir is passed, pip will unnecessarily archive
+                # the entire source directory
                 download_dir = None
             elif ireq.link and ireq.link.is_vcs:
                 # No download_dir for VCS sources.  This also works around pip
@@ -229,37 +216,26 @@ class PyPIRepository(BaseRepository):
                 download_dir = None
             else:
                 download_dir = self._get_download_path(ireq)
-                if not os.path.isdir(download_dir):
-                    os.makedirs(download_dir)
-            if PIP_VERSION[:2] <= (20, 2) and not os.path.isdir(
-                self._wheel_download_dir
-            ):
-                os.makedirs(self._wheel_download_dir)
+                os.makedirs(download_dir, exist_ok=True)
 
             with global_tempdir_manager():
                 wheel_cache = WheelCache(self._cache_dir, self.options.format_control)
-                prev_tracker = os.environ.get("PIP_REQ_TRACKER")
-                try:
-                    self._dependencies_cache[ireq] = self.resolve_reqs(
-                        download_dir, ireq, wheel_cache
-                    )
-                finally:
-                    if "PIP_REQ_TRACKER" in os.environ:
-                        if prev_tracker:
-                            os.environ["PIP_REQ_TRACKER"] = prev_tracker
-                        else:
-                            del os.environ["PIP_REQ_TRACKER"]
+                self._dependencies_cache[ireq] = self.resolve_reqs(
+                    download_dir, ireq, wheel_cache
+                )
 
         return self._dependencies_cache[ireq]
 
-    def copy_ireq_dependencies(self, source, dest):
+    def copy_ireq_dependencies(
+        self, source: InstallRequirement, dest: InstallRequirement
+    ) -> None:
         try:
             self._dependencies_cache[dest] = self._dependencies_cache[source]
         except KeyError:
             # `source` may not be in cache yet.
             pass
 
-    def _get_project(self, ireq):
+    def _get_project(self, ireq: InstallRequirement) -> Any:
         """
         Return a dict of a project info from PyPI JSON API for a given
         InstallRequirement. Return None on HTTP/JSON error or if a package
@@ -272,15 +248,11 @@ class PyPIRepository(BaseRepository):
             for index_url in self.finder.search_scope.index_urls
         )
         for package_index in package_indexes:
-            url = "{url}/{name}/json".format(url=package_index.pypi_url, name=ireq.name)
+            url = f"{package_index.pypi_url}/{ireq.name}/json"
             try:
                 response = self.session.get(url)
             except RequestException as e:
-                log.debug(
-                    "Fetch package info from PyPI failed: {url}: {e}".format(
-                        url=url, e=e
-                    )
-                )
+                log.debug(f"Fetch package info from PyPI failed: {url}: {e}")
                 continue
 
             # Skip this PyPI server, because there is no package
@@ -291,16 +263,12 @@ class PyPIRepository(BaseRepository):
             try:
                 data = response.json()
             except ValueError as e:
-                log.debug(
-                    "Cannot parse JSON response from PyPI: {url}: {e}".format(
-                        url=url, e=e
-                    )
-                )
+                log.debug(f"Cannot parse JSON response from PyPI: {url}: {e}")
                 continue
             return data
         return None
 
-    def _get_download_path(self, ireq):
+    def _get_download_path(self, ireq: InstallRequirement) -> str:
         """
         Determine the download dir location in a way which avoids name
         collisions.
@@ -309,12 +277,13 @@ class PyPIRepository(BaseRepository):
             salt = hashlib.sha224(ireq.link.url_without_fragment.encode()).hexdigest()
             # Nest directories to avoid running out of top level dirs on some FS
             # (see pypi _get_cache_path_parts, which inspired this)
-            salt = [salt[:2], salt[2:4], salt[4:6], salt[6:]]
-            return os.path.join(self._download_dir, *salt)
+            return os.path.join(
+                self._download_dir, salt[:2], salt[2:4], salt[4:6], salt[6:]
+            )
         else:
             return self._download_dir
 
-    def get_hashes(self, ireq):
+    def get_hashes(self, ireq: InstallRequirement) -> Set[str]:
         """
         Given an InstallRequirement, return a set of hashes that represent all
         of the files for a given requirement. Unhashable requirements return an
@@ -342,9 +311,9 @@ class PyPIRepository(BaseRepository):
                 return {self._get_file_hash(cached_link)}
 
         if not is_pinned_requirement(ireq):
-            raise TypeError("Expected pinned requirement, got {}".format(ireq))
+            raise TypeError(f"Expected pinned requirement, got {ireq}")
 
-        log.debug("{}".format(ireq.name))
+        log.debug(ireq.name)
 
         with log.indentation():
             hashes = self._get_hashes_from_pypi(ireq)
@@ -354,7 +323,7 @@ class PyPIRepository(BaseRepository):
 
         return hashes
 
-    def _get_hashes_from_pypi(self, ireq):
+    def _get_hashes_from_pypi(self, ireq: InstallRequirement) -> Optional[Set[str]]:
         """
         Return a set of hashes from PyPI JSON API for a given InstallRequirement.
         Return None if fetching data is failed or missing digests.
@@ -373,9 +342,7 @@ class PyPIRepository(BaseRepository):
 
         try:
             hashes = {
-                "{algo}:{digest}".format(
-                    algo=FAVORITE_HASH, digest=file_["digests"][FAVORITE_HASH]
-                )
+                f"{FAVORITE_HASH}:{file_['digests'][FAVORITE_HASH]}"
                 for file_ in release_files
                 if file_["packagetype"] in self.HASHABLE_PACKAGE_TYPES
             }
@@ -385,7 +352,7 @@ class PyPIRepository(BaseRepository):
 
         return hashes
 
-    def _get_hashes_from_files(self, ireq):
+    def _get_hashes_from_files(self, ireq: InstallRequirement) -> Set[str]:
         """
         Return a set of hashes for all release files of a given InstallRequirement.
         """
@@ -393,9 +360,9 @@ class PyPIRepository(BaseRepository):
         # pin, these will represent all of the files that could possibly
         # satisfy this constraint.
         all_candidates = self.find_all_candidates(ireq.name)
-        candidates_by_version = lookup_table(all_candidates, key=lambda c: c.version)
+        candidates_by_version = lookup_table(all_candidates, key=candidate_version)
         matching_versions = list(
-            ireq.specifier.filter((candidate.version for candidate in all_candidates))
+            ireq.specifier.filter(candidate.version for candidate in all_candidates)
         )
         matching_candidates = candidates_by_version[matching_versions[0]]
 
@@ -403,19 +370,18 @@ class PyPIRepository(BaseRepository):
             self._get_file_hash(candidate.link) for candidate in matching_candidates
         }
 
-    def _get_file_hash(self, link):
-        log.debug("Hashing {}".format(link.show_url))
+    def _get_file_hash(self, link: Link) -> str:
+        log.debug(f"Hashing {link.show_url}")
         h = hashlib.new(FAVORITE_HASH)
         with open_local_or_remote_file(link, self.session) as f:
             # Chunks to iterate
-            chunks = iter(lambda: f.stream.read(FILE_CHUNK_SIZE), b"")
+            chunks = iter(lambda: cast(bytes, f.stream.read(FILE_CHUNK_SIZE)), b"")
 
             # Choose a context manager depending on verbosity
+            context_manager: ContextManager[Iterator[bytes]]
             if log.verbosity >= 1:
                 iter_length = f.size / FILE_CHUNK_SIZE if f.size else None
-                bar_template = "{prefix}  |%(bar)s| %(info)s".format(
-                    prefix=" " * log.current_indent
-                )
+                bar_template = f"{' ' * log.current_indent}  |%(bar)s| %(info)s"
                 context_manager = progressbar(
                     chunks,
                     length=iter_length,
@@ -435,7 +401,7 @@ class PyPIRepository(BaseRepository):
         return ":".join([FAVORITE_HASH, h.hexdigest()])
 
     @contextmanager
-    def allow_all_wheels(self):
+    def allow_all_wheels(self) -> Iterator[None]:
         """
         Monkey patches pip.Wheel to allow wheels from all platforms and Python versions.
 
@@ -443,11 +409,11 @@ class PyPIRepository(BaseRepository):
         the previous non-patched calls will interfere.
         """
 
-        def _wheel_supported(self, tags=None):
+        def _wheel_supported(self: Wheel, tags: List[Tag]) -> bool:
             # Ignore current platform. Support everything.
             return True
 
-        def _wheel_support_index_min(self, tags=None):
+        def _wheel_support_index_min(self: Wheel, tags: List[Tag]) -> int:
             # All wheels are equal priority for sorting.
             return 0
 
@@ -466,7 +432,7 @@ class PyPIRepository(BaseRepository):
             Wheel.support_index_min = original_support_index_min
             self._available_candidates_cache = original_cache
 
-    def _setup_logging(self):
+    def _setup_logging(self) -> None:
         """
         Setup pip's logger. Ensure pip is verbose same as pip-tools and sync
         pip's log stream with LogContext.stream.
@@ -482,6 +448,7 @@ class PyPIRepository(BaseRepository):
         logger = logging.getLogger()
         for handler in logger.handlers:
             if handler.name == "console":  # pragma: no branch
+                assert isinstance(handler, logging.StreamHandler)
                 handler.stream = log.stream
                 break
         else:  # pragma: no cover
@@ -496,7 +463,7 @@ class PyPIRepository(BaseRepository):
 
 
 @contextmanager
-def open_local_or_remote_file(link, session):
+def open_local_or_remote_file(link: Link, session: Session) -> Iterator[FileStream]:
     """
     Open local or remote file for reading.
 
@@ -511,7 +478,7 @@ def open_local_or_remote_file(link, session):
         # Local URL
         local_path = url_to_path(url)
         if os.path.isdir(local_path):
-            raise ValueError("Cannot open directory for read: {}".format(url))
+            raise ValueError(f"Cannot open directory for read: {url}")
         else:
             st = os.stat(local_path)
             with open(local_path, "rb") as local_file:
@@ -522,6 +489,7 @@ def open_local_or_remote_file(link, session):
         response = session.get(url, headers=headers, stream=True)
 
         # Content length must be int or None
+        content_length: Optional[int]
         try:
             content_length = int(response.headers["content-length"])
         except (ValueError, KeyError, TypeError):
@@ -531,3 +499,7 @@ def open_local_or_remote_file(link, session):
             yield FileStream(stream=response.raw, size=content_length)
         finally:
             response.close()
+
+
+def candidate_version(candidate: InstallationCandidate) -> _BaseVersion:
+    return candidate.version
