@@ -1,15 +1,16 @@
-# coding: utf-8
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import itertools
 import os
 import shlex
 import sys
+from typing import List, Optional, Tuple, cast
 
+import click
 from pip._internal.commands import create_command
+from pip._internal.commands.install import InstallCommand
+from pip._internal.index.package_finder import PackageFinder
 from pip._internal.utils.misc import get_installed_distributions
 
-from .. import click, sync
+from .. import sync
 from .._compat import parse_requirements
 from ..exceptions import PipToolsError
 from ..logging import log
@@ -19,7 +20,7 @@ from ..utils import flat_map
 DEFAULT_REQUIREMENTS_FILE = "requirements.txt"
 
 
-@click.command()
+@click.command(context_settings={"help_option_names": ("-h", "--help")})
 @click.version_option()
 @click.option(
     "-a",
@@ -39,19 +40,10 @@ DEFAULT_REQUIREMENTS_FILE = "requirements.txt"
     "--find-links",
     multiple=True,
     help="Look for archives in this directory or on this HTML page",
-    envvar="PIP_FIND_LINKS",
 )
+@click.option("-i", "--index-url", help="Change index URL (defaults to PyPI)")
 @click.option(
-    "-i",
-    "--index-url",
-    help="Change index URL (defaults to PyPI)",
-    envvar="PIP_INDEX_URL",
-)
-@click.option(
-    "--extra-index-url",
-    multiple=True,
-    help="Add additional index URL to search",
-    envvar="PIP_EXTRA_INDEX_URL",
+    "--extra-index-url", multiple=True, help="Add additional index URL to search"
 )
 @click.option(
     "--trusted-host",
@@ -63,7 +55,8 @@ DEFAULT_REQUIREMENTS_FILE = "requirements.txt"
     is_flag=True,
     help="Ignore package index (only looking at --find-links URLs instead)",
 )
-@click.option("-q", "--quiet", default=False, is_flag=True, help="Give less output")
+@click.option("-v", "--verbose", count=True, help="Show more output")
+@click.option("-q", "--quiet", count=True, help="Give less output")
 @click.option(
     "--user", "user_only", is_flag=True, help="Restrict attention to user directory"
 )
@@ -76,22 +69,25 @@ DEFAULT_REQUIREMENTS_FILE = "requirements.txt"
 @click.argument("src_files", required=False, type=click.Path(exists=True), nargs=-1)
 @click.option("--pip-args", help="Arguments to pass directly to pip install.")
 def cli(
-    ask,
-    dry_run,
-    force,
-    find_links,
-    index_url,
-    extra_index_url,
-    trusted_host,
-    no_index,
-    quiet,
-    user_only,
-    cert,
-    client_cert,
-    src_files,
-    pip_args,
-):
+    ask: bool,
+    dry_run: bool,
+    force: bool,
+    find_links: Tuple[str, ...],
+    index_url: Optional[str],
+    extra_index_url: Tuple[str, ...],
+    trusted_host: Tuple[str, ...],
+    no_index: bool,
+    verbose: int,
+    quiet: int,
+    user_only: bool,
+    cert: Optional[str],
+    client_cert: Optional[str],
+    src_files: Tuple[str, ...],
+    pip_args: Optional[str],
+) -> None:
     """Synchronize virtual environment with requirements.txt."""
+    log.verbosity = verbose - quiet
+
     if not src_files:
         if os.path.exists(DEFAULT_REQUIREMENTS_FILE):
             src_files = (DEFAULT_REQUIREMENTS_FILE,)
@@ -112,7 +108,7 @@ def cli(
             log.error("ERROR: " + msg)
             sys.exit(2)
 
-    install_command = create_command("install")
+    install_command = cast(InstallCommand, create_command("install"))
     options, _ = install_command.parse_args([])
     session = install_command._build_session(options)
     finder = install_command._build_package_finder(options=options, session=session)
@@ -124,30 +120,32 @@ def cli(
     )
 
     try:
-        requirements = sync.merge(requirements, ignore_conflicts=force)
+        merged_requirements = sync.merge(requirements, ignore_conflicts=force)
     except PipToolsError as e:
         log.error(str(e))
         sys.exit(2)
 
     installed_dists = get_installed_distributions(skip=[], user_only=user_only)
-    to_install, to_uninstall = sync.diff(requirements, installed_dists)
+    to_install, to_uninstall = sync.diff(merged_requirements, installed_dists)
 
-    install_flags = _compose_install_flags(
-        finder,
-        no_index=no_index,
-        index_url=index_url,
-        extra_index_url=extra_index_url,
-        trusted_host=trusted_host,
-        find_links=find_links,
-        user_only=user_only,
-        cert=cert,
-        client_cert=client_cert,
-    ) + shlex.split(pip_args or "")
+    install_flags = (
+        _compose_install_flags(
+            finder,
+            no_index=no_index,
+            index_url=index_url,
+            extra_index_url=extra_index_url,
+            trusted_host=trusted_host,
+            find_links=find_links,
+            user_only=user_only,
+            cert=cert,
+            client_cert=client_cert,
+        )
+        + shlex.split(pip_args or "")
+    )
     sys.exit(
         sync.sync(
             to_install,
             to_uninstall,
-            verbose=(not quiet),
             dry_run=dry_run,
             install_flags=install_flags,
             ask=ask,
@@ -156,16 +154,16 @@ def cli(
 
 
 def _compose_install_flags(
-    finder,
-    no_index=False,
-    index_url=None,
-    extra_index_url=None,
-    trusted_host=None,
-    find_links=None,
-    user_only=False,
-    cert=None,
-    client_cert=None,
-):
+    finder: PackageFinder,
+    no_index: bool,
+    index_url: Optional[str],
+    extra_index_url: Tuple[str, ...],
+    trusted_host: Tuple[str, ...],
+    find_links: Tuple[str, ...],
+    user_only: bool,
+    cert: Optional[str],
+    client_cert: Optional[str],
+) -> List[str]:
     """
     Compose install flags with the given finder and CLI options.
     """
@@ -174,7 +172,7 @@ def _compose_install_flags(
     # Build --index-url/--extra-index-url/--no-index
     if no_index:
         result.append("--no-index")
-    elif index_url:
+    elif index_url is not None:
         result.extend(["--index-url", index_url])
     elif finder.index_urls:
         finder_index_url = finder.index_urls[0]
@@ -185,15 +183,15 @@ def _compose_install_flags(
     else:
         result.append("--no-index")
 
-    for extra_index in extra_index_url or []:
+    for extra_index in extra_index_url:
         result.extend(["--extra-index-url", extra_index])
 
     # Build --trusted-hosts
-    for host in itertools.chain(trusted_host or [], finder.trusted_hosts):
+    for host in itertools.chain(trusted_host, finder.trusted_hosts):
         result.extend(["--trusted-host", host])
 
     # Build --find-links
-    for link in itertools.chain(find_links or [], finder.find_links):
+    for link in itertools.chain(find_links, finder.find_links):
         result.extend(["--find-links", link])
 
     # Build format controls --no-binary/--only-binary
@@ -208,10 +206,10 @@ def _compose_install_flags(
     if user_only:
         result.append("--user")
 
-    if cert:
+    if cert is not None:
         result.extend(["--cert", cert])
 
-    if client_cert:
+    if client_cert is not None:
         result.extend(["--client-cert", client_cert])
 
     return result
