@@ -1,27 +1,28 @@
+import subprocess
 import sys
+from unittest import mock
 
-import mock
 import pytest
 
 from piptools.scripts.sync import DEFAULT_REQUIREMENTS_FILE, cli
 
-from .utils import invoke
-
 
 def test_run_as_module_sync():
     """piptools can be run as ``python -m piptools ...``."""
 
-    status, output = invoke([sys.executable, "-m", "piptools", "sync", "--help"])
+    result = subprocess.run(
+        [sys.executable, "-m", "piptools", "sync", "--help"],
+        stdout=subprocess.PIPE,
+        check=True,
+    )
 
     # Should have run pip-compile successfully.
-    output = output.decode("utf-8")
-    assert output.startswith("Usage:")
-    assert "Synchronize virtual environment with" in output
-    assert status == 0
+    assert result.stdout.startswith(b"Usage:")
+    assert b"Synchronize virtual environment with" in result.stdout
 
 
-@mock.patch("piptools.sync.check_call")
-def test_quiet_option(check_call, runner):
+@mock.patch("piptools.sync.run")
+def test_quiet_option(run, runner):
     """sync command can be run with `--quiet` or `-q` flag."""
 
     with open("requirements.txt", "w") as req_in:
@@ -32,13 +33,13 @@ def test_quiet_option(check_call, runner):
     assert out.exit_code == 0
 
     # for every call to pip ensure the `-q` flag is set
-    assert check_call.call_count == 2
-    for call in check_call.call_args_list:
+    assert run.call_count == 2
+    for call in run.call_args_list:
         assert "-q" in call[0][0]
 
 
-@mock.patch("piptools.sync.check_call")
-def test_quiet_option_when_up_to_date(check_call, runner):
+@mock.patch("piptools.sync.run")
+def test_quiet_option_when_up_to_date(run, runner):
     """
     Sync should output nothing when everything is up to date and quiet option is set.
     """
@@ -50,7 +51,7 @@ def test_quiet_option_when_up_to_date(check_call, runner):
 
     assert not out.stderr_bytes
     assert out.exit_code == 0
-    check_call.assert_not_called()
+    run.assert_not_called()
 
 
 def test_no_requirements_file(runner):
@@ -86,7 +87,7 @@ def test_force_files_with_dot_in_extension(runner):
     with open("requirements.in", "w") as req_in:
         req_in.write("six==1.10.0")
 
-    with mock.patch("piptools.sync.check_call"):
+    with mock.patch("piptools.sync.run"):
         out = runner.invoke(cli, ["requirements.in", "--force"])
 
     assert "WARNING: Some input files have the .in extension" in out.stderr
@@ -113,7 +114,7 @@ def test_merge_error(req_lines, should_raise, runner):
         for line in req_lines:
             req_in.write(line + "\n")
 
-    with mock.patch("piptools.sync.check_call"):
+    with mock.patch("piptools.sync.run"):
         out = runner.invoke(cli, ["-n"])
 
     if should_raise:
@@ -157,8 +158,8 @@ def test_merge_error(req_lines, should_raise, runner):
         ),
     ),
 )
-@mock.patch("piptools.sync.check_call")
-def test_pip_install_flags(check_call, cli_flags, expected_install_flags, runner):
+@mock.patch("piptools.sync.run")
+def test_pip_install_flags(run, cli_flags, expected_install_flags, runner):
     """
     Test the cli flags have to be passed to the pip install command.
     """
@@ -167,7 +168,7 @@ def test_pip_install_flags(check_call, cli_flags, expected_install_flags, runner
 
     runner.invoke(cli, cli_flags)
 
-    call_args = [call[0][0] for call in check_call.call_args_list]
+    call_args = [call[0][0] for call in run.call_args_list]
     called_install_options = [args[6:] for args in call_args if args[3] == "install"]
     assert called_install_options == [expected_install_flags], "Called args: {}".format(
         call_args
@@ -186,8 +187,8 @@ def test_pip_install_flags(check_call, cli_flags, expected_install_flags, runner
         ["--only-binary", ":all:"],
     ),
 )
-@mock.patch("piptools.sync.check_call")
-def test_pip_install_flags_in_requirements_file(check_call, runner, install_flags):
+@mock.patch("piptools.sync.run")
+def test_pip_install_flags_in_requirements_file(run, runner, install_flags):
     """
     Test the options from requirements.txt file pass to the pip install command.
     """
@@ -199,15 +200,13 @@ def test_pip_install_flags_in_requirements_file(check_call, runner, install_flag
     assert out.exit_code == 0, out
 
     # Make sure pip install command has expected options
-    call_args = [call[0][0] for call in check_call.call_args_list]
+    call_args = [call[0][0] for call in run.call_args_list]
     called_install_options = [args[6:] for args in call_args if args[3] == "install"]
-    assert called_install_options == [install_flags], "Called args: {}".format(
-        call_args
-    )
+    assert called_install_options == [install_flags], f"Called args: {call_args}"
 
 
-@mock.patch("piptools.sync.check_call")
-def test_sync_ask_declined(check_call, runner):
+@mock.patch("piptools.sync.run")
+def test_sync_ask_declined(run, runner):
     """
     Make sure nothing is installed if the confirmation is declined
     """
@@ -216,11 +215,11 @@ def test_sync_ask_declined(check_call, runner):
 
     runner.invoke(cli, ["--ask"], input="n\n")
 
-    check_call.assert_not_called()
+    run.assert_not_called()
 
 
-@mock.patch("piptools.sync.check_call")
-def test_sync_ask_accepted(check_call, runner):
+@mock.patch("piptools.sync.run")
+def test_sync_ask_accepted(run, runner):
     """
     Make sure pip is called when the confirmation is accepted (even if
     --dry-run is given)
@@ -230,7 +229,7 @@ def test_sync_ask_accepted(check_call, runner):
 
     runner.invoke(cli, ["--ask", "--dry-run"], input="y\n")
 
-    assert check_call.call_count == 2
+    assert run.call_count == 2
 
 
 def test_sync_dry_run_returns_non_zero_exit_code(runner):
