@@ -1,6 +1,5 @@
 import pytest
 
-from piptools._compat import PIP_VERSION
 from piptools.exceptions import NoCandidateFound
 from piptools.resolver import RequirementSummary, combine_install_requirements
 
@@ -325,40 +324,12 @@ def test_compile_failure_shows_provenance(resolver, from_line):
         ("test_package", "test_package", True),
         ("test_package==1.2.3", "test_package==1.2.3", True),
         ("test_package>=1.2.3", "test_package>=1.2.3", True),
-        pytest.param(
-            "test_package==1.2",
-            "test_package==1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
-        pytest.param(
-            "test_package>=1.2",
-            "test_package>=1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
+        ("test_package==1.2", "test_package==1.2.0", True),
+        ("test_package>=1.2", "test_package>=1.2.0", True),
         ("test_package[foo,bar]==1.2", "test_package[bar,foo]==1.2", True),
         ("test_package[foo,bar]>=1.2", "test_package[bar,foo]>=1.2", True),
-        pytest.param(
-            "test_package[foo,bar]==1.2",
-            "test_package[bar,foo]==1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
-        pytest.param(
-            "test_package[foo,bar]>=1.2",
-            "test_package[bar,foo]>=1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
+        ("test_package[foo,bar]==1.2", "test_package[bar,foo]==1.2.0", True),
+        ("test_package[foo,bar]>=1.2", "test_package[bar,foo]>=1.2.0", True),
         ("test_package", "other_test_package", False),
         ("test_package==1.2.3", "other_test_package==1.2.3", False),
         ("test_package==1.2.3", "test_package==1.2.4", False),
@@ -387,40 +358,12 @@ def test_RequirementSummary_equality(from_line, left_hand, right_hand, expected)
         ("test_package", "test_package", True),
         ("test_package==1.2.3", "test_package==1.2.3", True),
         ("test_package>=1.2.3", "test_package>=1.2.3", True),
-        pytest.param(
-            "test_package==1.2",
-            "test_package==1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
-        pytest.param(
-            "test_package>=1.2",
-            "test_package>=1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
+        ("test_package==1.2", "test_package==1.2.0", True),
+        ("test_package>=1.2", "test_package>=1.2.0", True),
         ("test_package[foo,bar]==1.2", "test_package[bar,foo]==1.2", True),
         ("test_package[foo,bar]>=1.2", "test_package[bar,foo]>=1.2", True),
-        pytest.param(
-            "test_package[foo,bar]==1.2",
-            "test_package[bar,foo]==1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
-        pytest.param(
-            "test_package[foo,bar]>=1.2",
-            "test_package[bar,foo]>=1.2.0",
-            True,
-            marks=pytest.mark.skipif(
-                PIP_VERSION[:2] < (20, 2), reason="Required only for pip>=20.2"
-            ),
-        ),
+        ("test_package[foo,bar]==1.2", "test_package[bar,foo]==1.2.0", True),
+        ("test_package[foo,bar]>=1.2", "test_package[bar,foo]>=1.2.0", True),
         ("test_package", "other_test_package", False),
         ("test_package==1.2.3", "other_test_package==1.2.3", False),
         ("test_package==1.2.3", "test_package==1.2.4", False),
@@ -441,3 +384,12 @@ def test_RequirementSummary_hash_equality(from_line, left_hand, right_hand, expe
     lh_summary = RequirementSummary(from_line(left_hand))
     rh_summary = RequirementSummary(from_line(right_hand))
     assert (hash(lh_summary) == hash(rh_summary)) is expected
+
+
+def test_requirement_summary_with_other_objects(from_line):
+    """
+    RequirementSummary should not be equal to any other object
+    """
+    requirement_summary = RequirementSummary(from_line("test_package==1.2.3"))
+    other_object = object()
+    assert requirement_summary != other_object
