@@ -1,27 +1,23 @@
-# coding: utf-8
-from __future__ import unicode_literals
-
 import logging
+import operator
 import os
+import shlex
 
 import pytest
-from pip._vendor import six
-from pip._vendor.six.moves import shlex_quote
 
 from piptools.scripts.compile import cli as compile_cli
 from piptools.utils import (
     as_tuple,
     dedup,
     flat_map,
-    force_text,
     format_requirement,
     format_specifier,
-    fs_str,
     get_compile_command,
     get_hashes_from_ireq,
     is_pinned_requirement,
     is_url_requirement,
-    name_from_req,
+    lookup_table,
+    lookup_table_from_tuples,
 )
 
 
@@ -87,7 +83,7 @@ def test_format_requirement_ireq_with_hashes_and_markers(from_line):
     assert format_requirement(ireq, marker, hashes=ireq_hashes) == expected
 
 
-def test_format_specifier(from_line):
+def test_format_specifier(from_line, from_editable):
     ireq = from_line("foo")
     assert format_specifier(ireq) == "<any>"
 
@@ -99,6 +95,11 @@ def test_format_specifier(from_line):
     ireq = from_line("foo~=1.1,<1.5,>1.2")
     assert format_specifier(ireq) == "~=1.1,>1.2,<1.5"
 
+    ireq = from_editable("git+https://github.com/django/django.git#egg=django")
+    assert format_specifier(ireq) == "<any>"
+    ireq = from_editable("file:///home/user/package")
+    assert format_specifier(ireq) == "<any>"
+
 
 def test_as_tuple(from_line):
     ireq = from_line("foo==1.1")
@@ -141,10 +142,10 @@ def test_get_hashes_from_ireq(from_line):
             }
         },
     )
-    expected = [
+    expected = {
         "sha256:d1d6729c85acea5423671382868627129432fba9a89ecbb248d8d1c7a9f01c67",
         "sha256:f5c056e8f62d45ba8215e5cb8f50dfccb198b4b9fbea8500674f3443e4689589",
-    ]
+    }
     assert get_hashes_from_ireq(ireq) == expected
 
 
@@ -197,35 +198,6 @@ def test_is_url_requirement_filename(caplog, from_line, line):
     assert is_url_requirement(ireq) is True
 
 
-def test_name_from_req(from_line):
-    ireq = from_line("django==1.8")
-    assert name_from_req(ireq.req) == "django"
-
-
-def test_name_from_req_with_project_name(from_line):
-    ireq = from_line("foo==1.8")
-    ireq.req.project_name = "bar"
-    assert name_from_req(ireq.req) == "bar"
-
-
-def test_fs_str():
-    assert fs_str("some path component/Something") == "some path component/Something"
-    assert isinstance(fs_str("whatever"), str)
-
-
-@pytest.mark.skipif(six.PY2, reason="Not supported in py2")
-def test_fs_str_with_bytes():
-    with pytest.raises(TypeError, match=r"^fs_str\(\) argument must not be bytes$"):
-        fs_str(b"whatever")
-
-
-@pytest.mark.parametrize(
-    ("value", "expected_text"), ((None, ""), (42, "42"), ("foo", "foo"), ("bãr", "bãr"))
-)
-def test_force_text(value, expected_text):
-    assert force_text(value) == expected_text
-
-
 @pytest.mark.parametrize(
     ("cli_args", "expected_command"),
     (
@@ -256,7 +228,6 @@ def test_force_text(value, expected_text):
         (["--pre"], "pip-compile --pre"),
         (["--allow-unsafe"], "pip-compile --allow-unsafe"),
         # Check negative flags
-        (["--no-index"], "pip-compile --no-index"),
         (["--no-emit-index-url"], "pip-compile --no-emit-index-url"),
         (["--no-emit-trusted-host"], "pip-compile --no-emit-trusted-host"),
         (["--no-annotate"], "pip-compile --no-annotate"),
@@ -264,7 +235,6 @@ def test_force_text(value, expected_text):
         # Check that default values will be removed from the command
         (["--emit-trusted-host"], "pip-compile"),
         (["--annotate"], "pip-compile"),
-        (["--index"], "pip-compile"),
         (["--emit-index-url"], "pip-compile"),
         (["--max-rounds=10"], "pip-compile"),
         (["--build-isolation"], "pip-compile"),
@@ -333,10 +303,9 @@ def test_get_compile_command_with_files(tmpdir_cwd, filename):
 
     args = [path, "--output-file", "requirements.txt"]
     with compile_cli.make_context("pip-compile", args) as ctx:
-        assert get_compile_command(
-            ctx
-        ) == "pip-compile --output-file=requirements.txt {src_file}".format(
-            src_file=shlex_quote(path)
+        assert (
+            get_compile_command(ctx)
+            == f"pip-compile --output-file=requirements.txt {shlex.quote(path)}"
         )
 
 
@@ -366,3 +335,35 @@ def test_get_compile_command_sort_args(tmpdir_cwd):
             "--no-annotate --no-emit-index-url --no-emit-trusted-host "
             "requirements.in setup.py"
         )
+
+
+@pytest.mark.parametrize(
+    "tuples",
+    (
+        (("f", "foo"), ("b", "bar"), ("b", "baz"), ("q", "qux"), ("q", "quux")),
+        iter((("f", "foo"), ("b", "bar"), ("b", "baz"), ("q", "qux"), ("q", "quux"))),
+    ),
+)
+def test_lookup_table_from_tuples(tuples):
+    expected = {"b": {"bar", "baz"}, "f": {"foo"}, "q": {"quux", "qux"}}
+    assert lookup_table_from_tuples(tuples) == expected
+
+
+@pytest.mark.parametrize(
+    ("values", "key"),
+    (
+        (("foo", "bar", "baz", "qux", "quux"), operator.itemgetter(0)),
+        (iter(("foo", "bar", "baz", "qux", "quux")), operator.itemgetter(0)),
+    ),
+)
+def test_lookup_table(values, key):
+    expected = {"b": {"bar", "baz"}, "f": {"foo"}, "q": {"quux", "qux"}}
+    assert lookup_table(values, key) == expected
+
+
+def test_lookup_table_from_tuples_with_empty_values():
+    assert lookup_table_from_tuples(()) == {}
+
+
+def test_lookup_table_with_empty_values():
+    assert lookup_table((), operator.itemgetter(0)) == {}
